import portal.tools.logging

from flask import Flask
from flask_cors import CORS
from flask_jwt_extended import JWTManager
from flask_oauthlib.client import OAuth

from portal.config import CONFIGURATIONS
from portal.tools.paths import ROOT_DIR
from flask_sqlalchemy import SQLAlchemy

import os
from storage import Storage

CORS_ORIGINS = [r'http://localhost:4200/*', 'https://*.pstanko.net/*']

db = SQLAlchemy()
jwt = JWTManager()
oauth = OAuth()
cors = CORS(resources=r'*', origins=CORS_ORIGINS, supports_credentials=True)
storage = Storage()


def config_app(flask_app):
    """Configures the Flask app based on production stage."""
    config_type = os.environ.get('PORTAL_CONFIG_TYPE', 'dev')
    config_object = CONFIGURATIONS[config_type]
    if config_object is None:
        raise RuntimeError(f"There is no config object for: {config_type}")
    flask_app.config.from_object(config_object)
    flask_app.config.from_pyfile(ROOT_DIR / 'portal.local.cfg', silent=True)
    flask_app.config.from_envvar('PORTAL_CONFIG', silent=True)
    flask_app.config['PORTAL_ENV'] = config_type
    # app.logger.removeHandler(default_handler) # from Flask release 1.0, not yet available


def configure_storage(app):
    storage_config = dict(
        test_files_dir=app.config.get('PORTAL_STORAGE_TEST_FILES_DIR'),
        submissions_dir=app.config.get('PORTAL_STORAGE_SUBMISSIONS_DIR'),
        workspace_dir=app.config.get('PORTAL_STORAGE_WORKSPACE_DIR')
    )
    storage.init_storage(**storage_config)


def create_app():
    app = Flask('portal')
    # app configuration
    config_app(app)
    configure_storage(app)
    # database bind to app
    db.init_app(app)
    # init the jwt
    jwt.init_app(app)
    # init the oauth client
    oauth.init_app(app)
    # init CORS
    cors.init_app(app)
    return app


import portal.database.models
